{% extends 'wildewidgets/block.html' %}
{% load wildewidgets %}


{% block block_content %}
{% for widget in entries %}
  
  <div class="{% if not forloop.first %}mt-5{% endif %}">
  <a id="{{ widget.get_title.get_title|slugify }}"></a>
      {% wildewidgets widget.get_title %}
  </div>

  <div class="{{block_name}}__entry {{entry_css_class}}">
    {% wildewidgets widget %}
  </div>
{% endfor %}
{% endblock %}

