# Generated by Django 3.0.8 on 2020-12-17 21:05

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Measurement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128)),
                ('time', models.DateTimeField(help_text='The date and time the measurement was made.', verbose_name='Time')),
                ('pressure', models.DecimalField(decimal_places=2, max_digits=8)),
                ('temperature', models.DecimalField(decimal_places=2, max_digits=8)),
                ('restricted', models.BooleanField(default=False)),
                ('open', models.BooleanField(default=True)),
            ],
        ),
    ]
